@@ -1,10 +1,5 @@
-# pylint: disable=too-many-lines
-# pylint: disable=too-many-instance-attributes
-# pylint: disable=invalid-name
-# pylint: disable=attribute-defined-outside-init
-"""
-Continuous multivariate probability distributions.
-"""
+"""Continuous multivariate probability distributions."""
+
 from copy import copy
 
 import numpy as np
@@ -15,12 +10,12 @@ except ImportError:
     pass
 from scipy import stats
 
-from .beta import Beta  # pylint: disable=no-name-in-module
-from .normal import Normal  # pylint: disable=no-name-in-module
-from .distributions_multivariate import Continuous
-from ..internal.distribution_helper import all_not_none
-from ..internal.plot_helper_multivariate import plot_dirichlet, plot_mvnormal
-from ..internal.plot_helper import check_inside_notebook, get_slider
+from preliz.distributions.beta import Beta
+from preliz.distributions.distributions_multivariate import Continuous
+from preliz.distributions.normal import Normal
+from preliz.internal.distribution_helper import all_not_none
+from preliz.internal.plot_helper import check_inside_notebook, get_slider
+from preliz.internal.plot_helper_multivariate import plot_dirichlet, plot_mvnormal
 
 eps = np.finfo(float).eps
 
@@ -90,11 +85,18 @@ class Dirichlet(Continuous):
     def _fit_mle(self, sample, **kwargs):
         raise NotImplementedError
 
+    def mode(self):
+        return (
+            (self.alpha - 1) / (np.sum(self.alpha) - len(self.alpha))
+            if np.all(self.alpha > 1)
+            else None
+        )
+
     def plot_pdf(
         self,
         marginals=True,
         pointinterval=False,
-        interval="hdi",
+        interval=None,
         levels=None,
         support="full",
         legend="title",
@@ -103,6 +105,7 @@ class Dirichlet(Continuous):
     ):
         """
         Plot the pdf of the marginals or the joint pdf of the simplex.
+
         The joint representation is only available for a dirichlet with an alpha of length 3.
 
         Parameters
@@ -114,8 +117,9 @@ class Dirichlet(Continuous):
             Whether to include a plot of the quantiles. Defaults to False. If True the default is to
             plot the median and two interquantile ranges.
         interval : str
-            Type of interval. Available options are highest density interval `"hdi"` (default),
+            Type of interval. Available options are highest density interval `"hdi"`,
         equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+        Defaults to the value in rcParams["stats.ci_kind"].
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -142,7 +146,7 @@ class Dirichlet(Continuous):
     def plot_cdf(
         self,
         pointinterval=False,
-        interval="hdi",
+        interval=None,
         levels=None,
         support="full",
         legend="title",
@@ -158,8 +162,9 @@ class Dirichlet(Continuous):
             Whether to include a plot of the quantiles. Defaults to False. If True the default is to
             plot the median and two interquantile ranges.
         interval : str
-            Type of interval. Available options are highest density interval `"hdi"` (default),
+            Type of interval. Available options are highest density interval `"hdi"`,
         equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+        Defaults to the value in rcParams["stats.ci_kind"].
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -186,7 +191,7 @@ class Dirichlet(Continuous):
     def plot_ppf(
         self,
         pointinterval=False,
-        interval="hdi",
+        interval=None,
         levels=None,
         legend="title",
         figsize=None,
@@ -201,8 +206,9 @@ class Dirichlet(Continuous):
             Whether to include a plot of the quantiles. Defaults to False. If True the default is to
             plot the median and two interquantile ranges.
         interval : str
-            Type of interval. Available options are highest density interval `"hdi"` (default),
+            Type of interval. Available options are highest density interval `"hdi"`,
         equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+        Defaults to the value in rcParams["stats.ci_kind"].
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -228,12 +234,13 @@ class Dirichlet(Continuous):
         kind="pdf",
         xy_lim="both",
         pointinterval=True,
-        interval="hdi",
+        interval=None,
         levels=None,
+        legend="title",
         figsize=None,
     ):
         """
-        Interactive exploration of parameters
+        Interactive exploration of parameters.
 
         Parameters
         ----------
@@ -249,20 +256,22 @@ class Dirichlet(Continuous):
             Whether to include a plot of the quantiles. Defaults to False.
             If `True` the default is to plot the median and two inter-quantiles ranges.
         interval : str
-            Type of interval. Available options are the highest density interval `"hdi"` (default),
+            Type of interval. Available options are the highest density interval `"hdi"`,
             equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
             (in this last case nothing will be plotted).
+        legend : str
+            Whether to include a string with the distribution and its parameter as a ``"title"``
+            or not include them ``None``.
         figsize : tuple
             Size of the figure
         """
-
         check_inside_notebook()
 
         args = dict(zip(self.param_names, self.params))
-        self.__init__(**args)  # pylint: disable=unnecessary-dunder-call
+        self.__init__(**args)
         if kind == "pdf":
             w_checkbox_marginals = widgets.Checkbox(
                 value=True,
@@ -282,7 +291,7 @@ class Dirichlet(Continuous):
             if kind == "pdf":
                 marginals = args.pop("marginals")
             params = {"alpha": np.asarray(list(args.values()), dtype=float)}
-            self.__init__(**params)  # pylint: disable=unnecessary-dunder-call
+            self.__init__(**params)
             if kind == "pdf":
                 plot_dirichlet(
                     self,
@@ -292,6 +301,7 @@ class Dirichlet(Continuous):
                     interval,
                     levels,
                     "full",
+                    legend,
                     figsize,
                     None,
                     xy_lim,
@@ -332,7 +342,7 @@ class MvNormal(Continuous):
 
     .. math::
 
-       f(x \mid \pi, T) =
+       f(x \mid \mu, T) =
            \frac{|T|^{1/2}}{(2\pi)^{k/2}}
            \exp\left\{ -\frac{1}{2} (x-\mu)^{\prime} T (x-\mu) \right\}
 
@@ -361,7 +371,7 @@ class MvNormal(Continuous):
 
     .. math::
 
-        \Tau = \Sigma^{-1}
+        T = \Sigma^{-1}
 
     Parameters
     ----------
@@ -420,11 +430,14 @@ class MvNormal(Continuous):
     def _fit_mle(self, sample, **kwargs):
         raise NotImplementedError
 
+    def mode(self):
+        return self.mu
+
     def plot_pdf(
         self,
         marginals=True,
         pointinterval=False,
-        interval="hdi",
+        interval=None,
         levels=None,
         support="full",
         legend="title",
@@ -432,7 +445,8 @@ class MvNormal(Continuous):
         ax=None,
     ):
         """
-        Plot the pdf of the marginals or the joint pdf
+        Plot the pdf of the marginals or the joint pdf.
+
         The joint representation is only available for a 2D Multivariate Normal.
 
         Parameters
@@ -444,8 +458,9 @@ class MvNormal(Continuous):
             Whether to include a plot of the quantiles. Defaults to False. If True the default is to
             plot the median and two interquantiles ranges.
         interval : str
-            Type of interval. Available options are highest density interval `"hdi"` (default),
+            Type of interval. Available options are highest density interval `"hdi"`,
         equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+        Defaults to the value in rcParams["stats.ci_kind"].
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -472,7 +487,7 @@ class MvNormal(Continuous):
     def plot_cdf(
         self,
         pointinterval=False,
-        interval="hdi",
+        interval=None,
         levels=None,
         support="full",
         legend="title",
@@ -488,8 +503,9 @@ class MvNormal(Continuous):
             Whether to include a plot of the quantiles. Defaults to False. If True the default is to
             plot the median and two interquantiles ranges.
         interval : str
-            Type of interval. Available options are highest density interval `"hdi"` (default),
+            Type of interval. Available options are highest density interval `"hdi"`,
         equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+        Defaults to the value in rcParams["stats.ci_kind"].
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -516,7 +532,7 @@ class MvNormal(Continuous):
     def plot_ppf(
         self,
         pointinterval=False,
-        interval="hdi",
+        interval=None,
         levels=None,
         legend="title",
         figsize=None,
@@ -531,8 +547,9 @@ class MvNormal(Continuous):
             Whether to include a plot of the quantiles. Defaults to False. If True the default is to
             plot the median and two interquantiles ranges.
         interval : str
-            Type of interval. Available options are highest density interval `"hdi"` (default),
+            Type of interval. Available options are highest density interval `"hdi"`,
         equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+        Defaults to the value in rcParams["stats.ci_kind"].
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -558,12 +575,13 @@ class MvNormal(Continuous):
         kind="pdf",
         xy_lim="both",
         pointinterval=True,
-        interval="hdi",
+        interval=None,
         levels=None,
+        legend="title",
         figsize=None,
     ):
         """
-        Interactive exploration of parameters
+        Interactive exploration of parameters.
 
         Parameters
         ----------
@@ -579,21 +597,24 @@ class MvNormal(Continuous):
             Whether to include a plot of the quantiles. Defaults to False.
             If `True` the default is to plot the median and two inter-quantiles ranges.
         interval : str
-            Type of interval. Available options are the highest density interval `"hdi"` (default),
+            Type of interval. Available options are the highest density interval `"hdi"`,
             equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+            Defaults to the value in rcParams["stats.ci_kind"].
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
             (in this last case nothing will be plotted).
+        legend : str
+            Whether to include a string with the distribution and its parameter as a ``"title"``
+            or not include them ``None``.
         figsize : tuple
             Size of the figure
         """
-
         check_inside_notebook()
 
         args = dict(zip(self.param_names, self.params))
         cov, tau = args.get("cov", None), args.get("tau", None)
-        self.__init__(**args)  # pylint: disable=unnecessary-dunder-call
+        self.__init__(**args)
         if kind == "pdf":
             w_checkbox_marginals = widgets.Checkbox(
                 value=True,
@@ -613,7 +634,7 @@ class MvNormal(Continuous):
             if kind == "pdf":
                 marginals = args.pop("marginals")
             params = {"mu": np.asarray(list(args.values()), dtype=float), "cov": cov, "tau": tau}
-            self.__init__(**params)  # pylint: disable=unnecessary-dunder-call
+            self.__init__(**params)
             if kind == "pdf":
                 plot_mvnormal(
                     self,
@@ -623,10 +644,12 @@ class MvNormal(Continuous):
                     interval,
                     levels,
                     "full",
+                    legend,
                     figsize,
                     None,
                     xy_lim,
                 )
+
             elif kind == "cdf":
                 plot_mvnormal(
                     self,
@@ -636,6 +659,7 @@ class MvNormal(Continuous):
                     interval,
                     levels,
                     "full",
+                    legend,
                     figsize,
                     None,
                     xy_lim,
@@ -643,12 +667,13 @@ class MvNormal(Continuous):
             elif kind == "ppf":
                 plot_mvnormal(
                     self,
-                    "cdf",
+                    "ppf",
                     "marginals",
                     pointinterval,
                     interval,
                     levels,
                     None,
+                    legend,
                     figsize,
                     None,
                     xy_lim,
