@@ -1,8 +1,6 @@
-"""
-Parent classes for all families.
-"""
-# pylint: disable=no-member
-# pylint: disable=import-outside-toplevel
+"""Parent classes for all families."""
+
+import warnings
 from collections import namedtuple
 from copy import copy
 
@@ -12,20 +10,16 @@ except ImportError:
     pass
 import numpy as np
 
-from ..internal.plot_helper import (
-    plot_pdfpmf,
-    plot_cdf,
-    plot_ppf,
-    get_slider,
+from preliz.internal.distribution_helper import init_vals, valid_distribution, valid_scalar_params
+from preliz.internal.optimization import find_hdi, find_mode
+from preliz.internal.plot_helper import (
     check_inside_notebook,
+    get_slider,
+    plot_cdf,
+    plot_pdfpmf,
+    plot_ppf,
 )
-from ..internal.distribution_helper import (
-    init_vals,
-    valid_scalar_params,
-    valid_distribution,
-)
-
-from ..internal.optimization import optimize_hdi
+from preliz.internal.rcparams import rcParams
 
 
 class Distribution:
@@ -43,8 +37,18 @@ class Distribution:
         name = self.__class__.__name__
         if name in ["Truncated", "Censored", "Hurdle"]:
             name += self.dist.__class__.__name__
+        if name == "Mixture":
+            name = (
+                "Mixture"
+                + "".join(dict.fromkeys(dist.__class__.__name__ for dist in self.dist))
+                + "\n"
+            )
+
         if self.is_frozen:
-            bolded_name = "\033[1m" + name + "\033[0m"
+            if "Mixture" in name:
+                bolded_name = "\033[1m" + name.strip() + "\033[0m" + "\n"
+            else:
+                bolded_name = "\033[1m" + name + "\033[0m"
 
             description = "".join(
                 f"{n}={v:.3g}, "
@@ -52,7 +56,6 @@ class Distribution:
                 else f"{n}=[" + "".join(f"{vi:.3g}, " for vi in v).strip(", ") + "], "
                 for n, v in zip(self.param_names, self.params)
             ).strip(", ")
-
             return f"{bolded_name}({description})"
         else:
             return name
@@ -64,25 +67,28 @@ class Distribution:
         else:
             return None
 
-    def summary(self, mass=0.94, interval="hdi", fmt=".2f"):
+    def summary(self, mass=None, interval=None, fmt=".2f"):
         """
-        Namedtuple with the mean, median, standard deviation, and lower and upper bounds
-        of the equal-tailed interval.
+        Namedtuple with the mean, median, sd, and lower and upper bounds.
 
         Parameters
         ----------
         mass: float
-            Probability mass for the equal-tailed interval. Defaults to 0.94
+            Probability mass for the equal-tailed interval. Defaults to None,
+        which results in the value of rcParams["stats.ci_prob"] being used.
         interval : str or list-like
-            Type of interval. Available options are highest density interval `"hdi"` (default),
+            Type of interval. Available options are highest density interval `"hdi"`,
             equal tailed interval `"eti"` or arbitrary interval defined by a list-like object
-            with a pair of values.
+            with a pair of values. Defaults to the value in rcParams["stats.ci_kind"].
         fmt : str
             fmt used to represent results using f-string fmt for floats. Default to ".2f"
             i.e. 2 digits after the decimal point.
         """
         valid_distribution(self)
 
+        if interval is None:
+            interval = rcParams["stats.ci_kind"]
+
         if not isinstance(fmt, str):
             raise ValueError("Invalid format string.")
 
@@ -92,12 +98,17 @@ class Distribution:
                 name = "Truncated" + self.dist.__class__.__name__
             elif name == "Censored":
                 name = "Censored" + self.dist.__class__.__name__
+            elif name == "Mixture":
+                name = "Mixture" + "".join(
+                    dict.fromkeys(dist.__class__.__name__ for dist in self.dist)
+                )
+
             attr = namedtuple(name, ["mean", "median", "std", "lower", "upper"])
             mean = float(f"{self.mean():{fmt}}")
             median = float(f"{self.median():{fmt}}")
             std = float(f"{self.std():{fmt}}")
 
-            if isinstance(interval, (tuple, list, np.ndarray)):
+            if isinstance(interval, tuple | list | np.ndarray):
                 c_int = self.ppf(interval)
             elif interval == "hdi":
                 c_int = self.hdi(mass, fmt=fmt)
@@ -114,8 +125,8 @@ class Distribution:
         else:
             return None
 
-    def rvs(self, *args, **kwds):
-        """Random sample
+    def rvs(self, size=None, random_state=None):
+        """Random sample.
 
         Parameters
         ----------
@@ -124,9 +135,19 @@ class Distribution:
         random_state : {None, int, numpy.random.Generator, numpy.random.RandomState}
             Defaults to None
         """
-        return self.rvs(*args, **kwds)
+        raise NotImplementedError
 
-    def cdf(self, x, *args, **kwds):
+    def pdf(self, x):
+        """Probability density/mass function.
+
+        Parameters
+        ----------
+        x : array_like
+            Values on which to evaluate the pdf
+        """
+        raise NotImplementedError
+
+    def cdf(self, x):
         """Cumulative distribution function.
 
         Parameters
@@ -134,9 +155,9 @@ class Distribution:
         x : array_like
             Values on which to evaluate the cdf
         """
-        return self.cdf(x, *args, **kwds)
+        raise NotImplementedError
 
-    def ppf(self, q, *args, **kwds):
+    def ppf(self, q):
         """Percent point function (inverse of cdf).
 
         Parameters
@@ -144,31 +165,89 @@ class Distribution:
         x : array_like
             Values on which to evaluate the inverse of the cdf
         """
-        return self.ppf(q, *args, **kwds)
+        raise NotImplementedError
+
+    def logpdf(self, x):
+        """Log probability density/mass function.
+
+        Parameters
+        ----------
+        x : array_like
+            Values on which to evaluate the logpdf
+        """
+        raise NotImplementedError
+
+    def logcdf(self, x):
+        """Log cumulative distribution function.
+
+        Parameters
+        ----------
+        x : array_like
+            Values on which to evaluate the logcdf
+        """
+        return np.log(self.cdf(x))
+
+    def sf(self, x):
+        """Survival function (1 - cdf).
+
+        Parameters
+        ----------
+        x : array_like
+            Values on which to evaluate the sf
+        """
+        return 1 - self.cdf(x)
+
+    def logsf(self, x):
+        """Log survival function log(1 - cdf).
+
+        Parameters
+        ----------
+        x : array_like
+            Values on which to evaluate the logsf
+        """
+        return np.log(self.sf(x))
+
+    def isf(self, x):
+        """Inverse survival function (inverse of sf).
+
+        Parameters
+        ----------
+        x : array_like
+            Values on which to evaluate the inverse of the sf
+        """
+        return self.ppf(1 - np.array(x))
+
+    def entropy(self):
+        """Entropy."""
+        raise NotImplementedError
 
     def mean(self):
-        """Mean of the distribution."""
-        return self.mean()
+        """Mean."""
+        raise NotImplementedError
 
     def median(self):
-        """Median of the distribution."""
-        return self.median()
+        """Median."""
+        raise NotImplementedError
+
+    def mode(self):
+        """Mode."""
+        raise NotImplementedError
 
     def std(self):
-        """Standard deviation of the distribution."""
-        return self.std()
+        """Standard deviation."""  # noqa: D401
+        raise NotImplementedError
 
     def var(self):
-        """Variance of the distribution."""
-        return self.var()
+        """Variance."""
+        raise NotImplementedError
 
     def skewness(self):
-        """Skewness of the distribution."""
-        return self.stats(moment="s")
+        """Skewness."""
+        raise NotImplementedError
 
     def kurtosis(self):
-        """Kurtosis of the distribution"""
-        return self.stats(moments="k")
+        """Excess kurtosis."""
+        raise NotImplementedError
 
     def moments(self, types="mvsk"):
         """
@@ -181,7 +260,8 @@ class Distribution:
         types : str
             The type of moments to compute. Default is 'mvsk'
             where 'm' = mean, 'v' = variance, 's' = skewness, and 'k' = kurtosis.
-            Valid combinations are any subset of 'mvsk'.
+            To compute the standard deviation use 'd'
+            Valid combinations are any subset of 'mvdsk'.
         """
         moments = []
         for m_t in types:
@@ -203,26 +283,29 @@ class Distribution:
 
         return moments
 
-    def eti(self, mass=0.94, fmt=".2f"):
+    def eti(self, mass=None, fmt=".2f"):
         """Equal-tailed interval containing `mass`.
 
         Parameters
         ----------
         mass: float
-            Probability mass in the interval. Defaults to 0.94
+            Probability mass in the interval. Defaults to None,
+        which results in the value of rcParams["stats.ci_prob"] being used.
         fmt : str
             fmt used to represent results using f-string fmt for floats. Default to ".2f"
             i.e. 2 digits after the decimal point. Use `"none"` for no format.
         """
         valid_distribution(self)
 
+        if mass is None:
+            mass = rcParams["stats.ci_prob"]
+
         if not isinstance(fmt, str):
             raise ValueError("Invalid format string.")
 
         if valid_scalar_params(self):
             lower_tail, upper_tail = self.ppf([(1 - mass) / 2, 1 - (1 - mass) / 2])
-            if self.kind == "continuos" and fmt != "none":
-                print("hi!")
+            if self.kind == "continuous" and fmt != "none":
                 lower_tail = float(f"{lower_tail:{fmt}}")
                 upper_tail = float(f"{upper_tail:{fmt}}")
             elif self.kind == "discrete":
@@ -233,25 +316,32 @@ class Distribution:
         else:
             return None
 
-    def hdi(self, mass=0.94, fmt=".2f"):
+    def hdi(self, mass=None, fmt=".2f"):
         """Highest density interval containing `mass`.
 
         Parameters
         ----------
         mass: float
-            Probability mass in the interval. Defaults to 0.94
+            Probability mass in the interval. Defaults to None,
+        which results in the value of rcParams["stats.ci_prob"] being used.
         fmt : str
             fmt used to represent results using f-string fmt for floats. Default to ".2f"
             i.e. 2 digits after the decimal point. Use `"none"` for no format.
         """
         valid_distribution(self)
 
+        if mass is None:
+            mass = rcParams["stats.ci_prob"]
+
         if not isinstance(fmt, str):
             raise ValueError("Invalid format string.")
 
+        if self.__class__.__name__ == "Mixture":
+            warnings.warn("HDI may not be correct for multimodal distributions")
+
         if valid_scalar_params(self):
-            lower_tail, upper_tail = optimize_hdi(self, mass)
-            if self.kind == "continuos" and fmt != "none":
+            lower_tail, upper_tail = find_hdi(self, mass)
+            if self.kind == "continuous" and fmt != "none":
                 lower_tail = float(f"{lower_tail:{fmt}}")
                 upper_tail = float(f"{upper_tail:{fmt}}")
             return (lower_tail, upper_tail)
@@ -277,20 +367,76 @@ class Distribution:
 
             model = Model.get_context(error_if_none=False)
 
-            if model is None:
-                return getattr(pm_dists, self.__class__.__name__).dist(**self.params_dict, **kwargs)
+            if self.__class__.__name__ == "Hurdle":
+                preliz_name = self.__class__.__name__ + self.dist.__class__.__name__
             else:
-                return getattr(pm_dists, self.__class__.__name__)(
-                    name, **self.params_dict, **kwargs
-                )
-        except ImportError:
-            pass
+                preliz_name = self.__class__.__name__
+            pymc_class = getattr(pm_dists, preliz_name)
 
-        return None
+            if model is None:
+                if self.__class__.__name__ in ["Truncated", "Censored"]:
+                    pymc_dist = pymc_class.dist(
+                        self.dist.to_pymc(),
+                        lower=self.params_dict["lower"],
+                        upper=self.params_dict["upper"],
+                        **kwargs,
+                    )
+                elif self.__class__.__name__ == "Mixture":
+                    pymc_dist = pymc_class.dist(
+                        self.weights,
+                        [dist.to_pymc() for dist in self.dist],
+                        **kwargs,
+                    )
+                else:
+                    pymc_dist = pymc_class.dist(**self.params_dict, **kwargs)
+            elif self.__class__.__name__ in ["Truncated", "Censored"]:
+                pymc_dist = pymc_class(
+                    name,
+                    getattr(pm_dists, self.dist.__class__.__name__).dist(**self.dist.params_dict),
+                    lower=self.params_dict["lower"],
+                    upper=self.params_dict["upper"],
+                    **kwargs,
+                )
+            elif self.__class__.__name__ == "Mixture":
+                pymc_dist = pymc_class(
+                    name,
+                    self.weights,
+                    [
+                        getattr(pm_dists, dist.__class__.__name__).dist(**dist.params_dict)
+                        for dist in self.dist
+                    ],
+                    **kwargs,
+                )
+            else:
+                pymc_dist = pymc_class(name, **self.params_dict, **kwargs)
+
+            return pymc_dist
+
+        except ImportError:
+            raise ImportError("This function requires PyMC") from None
+
+    def to_bambi(self, **kwargs):
+        """
+        Convert the PreliZ distribution to a Bambi Prior.
+
+        kwargs : PyMC distributions properties
+            kwargs are used to specify properties such as shape or dims
+
+        Returns
+        -------
+        Bambi Prior
+        """
+        try:
+            from bambi import Prior
+
+            return Prior(self.__class__.__name__, **self.params_dict, **kwargs)
+
+        except ImportError:
+            raise ImportError("This function requires Bambi") from None
 
     def _check_endpoints(self, lower, upper, raise_error=True):
         """
-        Evaluate if the lower and upper values are in the support of the distribution
+        Evaluate if the lower and upper values are in the support of the distribution.
 
         Parameters
         ----------
@@ -330,7 +476,7 @@ class Distribution:
 
     def _finite_endpoints(self, support):
         """
-        Return finite endpoints even for unbounded distributions
+        Return finite endpoints even for unbounded distributions.
 
         Parameters
         ----------
@@ -340,6 +486,9 @@ class Distribution:
         if isinstance(support, tuple):
             lower_ep, upper_ep = support
         else:
+            if support not in ["restricted", "full"]:
+                raise ValueError("Allowed values for the support are 'restricted' or 'full' ")
+
             lower_ep, upper_ep = self.support
 
             if not np.isfinite(lower_ep) or support == "restricted":
@@ -349,18 +498,49 @@ class Distribution:
 
         return lower_ep, upper_ep
 
+    def xvals(self, support, n_points=None):
+        """
+        Provide x values in the support of the distribution.
+
+        This is useful for example when plotting.
+
+        Parameters
+        ----------
+        support : str
+            Available options are `"full"` or `"restricted"`.
+            If `"full"` the values will cover the entire support of the distribution if the boundary
+            is finite, or the quantiles 0.0001 or 0.9999, if infinite.
+            If `"restricted"` the values will cover the quantile 0.0001 to 0.9999.
+        n_points : int
+            Number of values to return. Defaults to 1000 for continuous distributions
+            and 200 for discrete ones.
+            For discrete distributions the returned values may be fewer
+            than `n_points` if the actual number of discrete values in the support of the
+            distribution is smaller than `n_points`.
+        """
+        lower_ep, upper_ep = self._finite_endpoints(support)
+
+        if self.kind == "continuous":
+            if n_points is None:
+                n_points = 1000
+            return _continuous_xvals(lower_ep, upper_ep, n_points)
+        else:
+            if n_points is None:
+                n_points = 200
+            return _discrete_xvals(lower_ep, upper_ep, n_points)
+
     def plot_pdf(
         self,
         moments=None,
         pointinterval=False,
-        interval="hdi",
+        interval=None,
         levels=None,
         support="restricted",
+        baseline=True,
         legend="legend",
-        color=None,
-        alpha=1,
         figsize=None,
         ax=None,
+        **kwargs,
     ):
         """
         Plot the  pdf (continuous) or pmf (discrete).
@@ -377,6 +557,7 @@ class Distribution:
         interval : str
             Type of interval. Available options are highest density interval `"hdi"` (default),
             equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+            Defaults to the value in rcParams["stats.ci_kind"].
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -384,19 +565,18 @@ class Distribution:
         support : str:
             If ``full`` use the finite end-points to set the limits of the plot. For unbounded
             end-points or if ``restricted`` use the 0.001 and 0.999 quantiles to set the limits.
+        baseline : bool
+            Whether to include a horizontal line at y=0.
         legend : str
             Whether to include a string with the distribution and its parameter as a ``"legend"`` a
             ``"title"`` or not include them ``None``.
-        color : str
-            Valid matplotlib color. Defaults to None, colors will be selected from the active
-            color cycle.
-        alpha : float
-            Transparency of the line. Defaults to 1 (no transparency).
         figsize : tuple
             Size of the figure
         ax : matplotlib axes
+        kwargs : keyword arguments
+            Additional keyword arguments passed to matplotlib plot function.
+            For example, ``color``, ``alpha``, ``linewidth``, etc.
         """
-
         if valid_scalar_params(self):
             return plot_pdfpmf(
                 self,
@@ -405,11 +585,11 @@ class Distribution:
                 interval,
                 levels,
                 support,
+                baseline,
                 legend,
-                color,
-                alpha,
                 figsize,
                 ax,
+                kwargs,
             )
         else:
             return None
@@ -418,14 +598,13 @@ class Distribution:
         self,
         moments=None,
         pointinterval=False,
-        interval="hdi",
+        interval=None,
         levels=None,
         support="restricted",
         legend="legend",
-        color=None,
-        alpha=1,
         figsize=None,
         ax=None,
+        **kwargs,
     ):
         """
         Plot the cumulative distribution function.
@@ -442,6 +621,7 @@ class Distribution:
         interval : str
             Type of interval. Available options are highest density interval `"hdi"` (default),
             equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+            Defaults to the value in rcParams["stats.ci_kind"].
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -449,17 +629,15 @@ class Distribution:
         support : str:
             If ``full`` use the finite end-points to set the limits of the plot. For unbounded
             end-points or if ``restricted`` use the 0.001 and 0.999 quantiles to set the limits.
-        color : str
-            Valid matplotlib color. Defaults to None, colors will be selected from the active
-            color cycle.
-        alpha : float
-            Transparency of the line. Defaults to 1 (no transparency).
         legend : str
             Whether to include a string with the distribution and its parameter as a ``"legend"`` a
             ``"title"`` or not include them ``None``.
         figsize : tuple
             Size of the figure
         ax : matplotlib axes
+        kwargs : keyword arguments
+            Additional keyword arguments passed to matplotlib plot function.
+            For example, ``color``, ``alpha``, ``linewidth``, etc.
         """
         if valid_scalar_params(self):
             return plot_cdf(
@@ -470,10 +648,9 @@ class Distribution:
                 levels,
                 support,
                 legend,
-                color,
-                alpha,
                 figsize,
                 ax,
+                kwargs,
             )
         else:
             return None
@@ -482,13 +659,12 @@ class Distribution:
         self,
         moments=None,
         pointinterval=False,
-        interval="hdi",
+        interval=None,
         levels=None,
         legend="legend",
-        color=None,
-        alpha=1,
         figsize=None,
         ax=None,
+        **kwargs,
     ):
         """
         Plot the quantile function.
@@ -505,6 +681,7 @@ class Distribution:
         interval : str
             Type of interval. Available options are highest density interval `"hdi"` (default),
             equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+            Defaults to the value in rcParams["stats.ci_kind"].
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -512,18 +689,13 @@ class Distribution:
         legend : str
             Whether to include a string with the distribution and its parameter as a ``"legend"`` a
             ``"title"`` or not include them ``None``.
-        color : str
-            Valid matplotlib color. Defaults to None, colors will be selected from the active
-            color cycle.
-        alpha : float
-            Transparency of the line. Defaults to 1 (no transparency).
         figsize : tuple
             Size of the figure
         ax : matplotlib axes
         """
         if valid_scalar_params(self):
             return plot_ppf(
-                self, moments, pointinterval, interval, levels, legend, color, alpha, figsize, ax
+                self, moments, pointinterval, interval, levels, legend, figsize, ax, kwargs
             )
         else:
             return None
@@ -533,12 +705,12 @@ class Distribution:
         kind="pdf",
         xy_lim="both",
         pointinterval=True,
-        interval="hdi",
+        interval=None,
         levels=None,
         figsize=None,
     ):
         """
-        Interactive exploration of distributions parameters
+        Interactive exploration of distributions parameters.
 
         Parameters
         ----------
@@ -556,6 +728,7 @@ class Distribution:
         interval : str
             Type of interval. Available options are highest density interval `"hdi"` (default),
             equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+            Defaults to the value in rcParams["stats.ci_kind"].
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -604,7 +777,7 @@ class Distribution:
             for name, value, support in zip(self.param_names, self.params, self.params_support):
                 sliders[name] = get_slider(name, value, *support)
 
-        def plot(**args):  # pylint: disable=inconsistent-return-statements
+        def plot(**args):
             if self.__class__.__name__ == "Categorical":
                 values = list(args.values())
                 args = {list(args.keys())[0].split("_")[0]: values}
@@ -652,69 +825,8 @@ class Continuous(Distribution):
         super().__init__()
         self.kind = "continuous"
 
-    def xvals(self, support, n_points=1000):
-        """Provide x values in the support of the distribution. This is useful for example when
-        plotting.
-
-        Parameters
-        ----------
-        support : str
-            Available options are `full` or `restricted`. If `full` the values will cover the entire
-            support of the distribution, if finite, or the quantiles 0.0001 or 0.9999, if infinite.
-            If `restricted` the values will cover the quantile 0.0001 to 0.9999.
-        n_points : int
-            Number of values to return.
-        """
-        half_n_points = int(n_points / 2)
-
-        if isinstance(support, tuple):
-            even = np.linspace(*support, n_points)
-            uneven = self.ppf(np.linspace(*self.cdf(support), n_points))
-        else:
-            lower_ep, upper_ep = self.support
-
-            if not np.isfinite(lower_ep) or support == "restricted":
-                lower_ep = 0.0001
-            if not np.isfinite(upper_ep) or support == "restricted":
-                upper_ep = 0.9999
-
-            even = np.linspace(*self.ppf([lower_ep, upper_ep]), half_n_points)
-            uneven = self.ppf(np.linspace(lower_ep, upper_ep, half_n_points))
-
-        return np.sort(np.concatenate([even, uneven]))
-
-    def _fit_mle(self, sample, **kwargs):
-        """
-        Estimate the parameters of the distribution from a sample by maximizing the likelihood.
-
-        Parameters
-        ----------
-        sample : array-like
-            a sample
-        kwargs : dict
-            keywords arguments passed to scipy.stats.rv_continuous.fit
-        """
-        raise NotImplementedError
-
-    def pdf(self, x, *args, **kwds):
-        """Probability density function at x.
-
-        Parameters
-        ----------
-        x : array_like
-            Values on which to evaluate the pdf
-        """
-        return self.pdf(x, *args, **kwds)
-
-    def logpdf(self, x, *args, **kwds):
-        """Probability mass function at x.
-
-        Parameters
-        ----------
-        x : array_like
-            Values on which to evaluate the pdf
-        """
-        return self.logpdf(x, *args, **kwds)
+    def mode(self):
+        return find_mode(self)
 
 
 class Discrete(Distribution):
@@ -724,95 +836,27 @@ class Discrete(Distribution):
         super().__init__()
         self.kind = "discrete"
 
-    def xvals(self, support, n_points=200):
-        """Provide x values in the support of the distribution. This is useful for example when
-        plotting.
-
-        Parameters
-        ----------
-        support : str
-            Available options are `full` or `restricted`. If `full` the values will cover the entire
-            support of the distribution, if finite, or the quantiles 0.0001 or 0.9999, if infinite.
-            If `restricted` the values will cover the quantile 0.0001 to 0.9999.
-        n_points : int
-            Number of values to return. The returned values may be fewer than `n_points` if
-            the actual number of discrete values in the support of the distribution is smaller than
-            `n_points`.
-        """
-        lower_ep, upper_ep = self._finite_endpoints(support)
-        return discrete_xvals(lower_ep, upper_ep, n_points)
-
-    def pdf(self, x, *args, **kwds):
-        """Probability mass function at x.
-
-        Parameters
-        ----------
-        x : array_like
-            Values on which to evaluate the pdf
-        """
-        return self.pdf(x, *args, **kwds)
-
-    def logpdf(self, x, *args, **kwds):
-        """Probability mass function at x.
-
-        Parameters
-        ----------
-        x : array_like
-            Values on which to evaluate the pdf
-        """
-        return self.logpdf(x, *args, **kwds)
-
 
 class DistributionTransformer(Distribution):
-    """Base class for distributions that transform other distributions"""
+    """Base class for distributions that transform other distributions."""
 
     def __init__(self):
         super().__init__()
-        self.kind = self.dist.kind
-
-    def xvals(self, support, n_points=None):
-        """Provide x values in the support of the distribution. This is useful for example when
-        plotting.
-
-        Parameters
-        ----------
-        support : str
-            Available options are `full` or `restricted`. If `full` the values will cover the entire
-            support of the distribution, if finite, or the quantiles 0.0001 or 0.9999, if infinite.
-            If `restricted` the values will cover the quantile 0.0001 to 0.9999.
-        n_points : int
-            Number of values to return. For discrete distributions the returned values may be fewer
-            than `n_points` if the actual number of discrete values in the support of the
-            distribution is smaller than `n_points`.
-        """
-        lower_ep, upper_ep = self._finite_endpoints(support)
-
-        if self.kind == "continuous":
-            if n_points is None:
-                n_points = 1000
-            return continuous_xvals(lower_ep, upper_ep, n_points)
-        else:
-            if n_points is None:
-                n_points = 200
-            return discrete_xvals(lower_ep, upper_ep, n_points)
-
-    def skewness(self):
-        """Skewness of the distribution. Not implemented."""
-        return NotImplemented
-
-    def kurtosis(self):
-        """Kurtosis of the distribution. Not implemented."""
-        return NotImplemented
+        if not isinstance(self.dist, list):
+            self.kind = self.dist.kind
 
 
-def continuous_xvals(lower_ep, upper_ep, n_points):
+def _continuous_xvals(lower_ep, upper_ep, n_points):
     return np.linspace(lower_ep, upper_ep, n_points)
 
 
-def discrete_xvals(lower_ep, upper_ep, n_points):
+def _discrete_xvals(lower_ep, upper_ep, n_points):
+    upper_ep = int(upper_ep)
+    lower_ep = int(lower_ep)
     range_x = upper_ep - lower_ep
     if range_x <= n_points:
         x_vals = np.arange(lower_ep, upper_ep + 1, dtype=int)
     else:
         x_vals = np.linspace(lower_ep, upper_ep + 1, n_points, dtype=int)
+
     return x_vals
