@@ -1,4 +1,6 @@
 import re
+from sys import modules
+
 import numpy as np
 
 eps = np.finfo(float).eps
@@ -10,7 +12,7 @@ def from_precision(precision):
 
 
 def to_precision(sigma):
-    precision = 1 / sigma**2
+    precision = 1 / (eps + sigma**2)
     return precision
 
 
@@ -40,10 +42,9 @@ def valid_scalar_params(self, check_frozen=True):
     if self.kind not in ["discrete", "continuous"]:
         return True
 
-    if (
-        all(isinstance(param, (int, float, np.int64)) for param in self.params)
-        or self.__class__.__name__ == "Categorical"
-    ):
+    if all(
+        isinstance(param, int | float | np.int64) for param in self.params
+    ) or self.__class__.__name__ in ["Categorical", "Mixture"]:
         return True
 
     raise ValueError("parameters must be integers or floats")
@@ -64,14 +65,38 @@ def process_extra(input_string):
         name = match[0]
         args = match[1].split(",")
         arg_dict = {}
-        for arg in args:
-            key, value = arg.split("=")
-            arg_dict[key.strip()] = float(value)
-        result_dict[name] = arg_dict
-
+        try:
+            for arg in args:
+                key, value = arg.split("=")
+                arg_dict[key.strip()] = float(value)
+            result_dict[name] = arg_dict
+        except ValueError:
+            pass
     return result_dict
 
 
+def num_skewness(dist):
+    mean = dist.mean()
+    std = dist.std()
+    x_values = dist.xvals("full")
+    pdf = dist.pdf(x_values)
+    if dist.kind == "discrete":
+        return np.sum(((x_values - mean) / std) ** 3 * pdf)
+    else:
+        return np.trapezoid(((x_values - mean) / std) ** 3 * pdf, x_values)
+
+
+def num_kurtosis(dist):
+    mean = dist.mean()
+    std = dist.std()
+    x_values = dist.xvals("full")
+    pdf = dist.pdf(x_values)
+    if dist.kind == "discrete":
+        return np.sum(((x_values - mean) / std) ** 4 * pdf) - 3
+    else:
+        return np.trapezoid(((x_values - mean) / std) ** 4 * pdf, x_values) - 3
+
+
 init_vals = {
     "AsymmetricLaplace": {"kappa": 1.0, "mu": 0.0, "b": 1.0},
     "Beta": {"alpha": 2, "beta": 2},
@@ -96,8 +121,9 @@ init_vals = {
     "Normal": {"mu": 0.0, "sigma": 1.0},
     "Pareto": {"alpha": 5, "m": 2.0},
     "Rice": {"nu": 2.0, "sigma": 1.0},
+    "ScaledInverseChiSquared": {"nu": 10, "tau2": 1.0},
     "SkewNormal": {"mu": 0.0, "sigma": 1, "alpha": 6.0},
-    "SkewStudentT": {"mu": 0.0, "sigma": 1, "a": 2.0, "b": 2.0},
+    "SkewStudentT": {"mu": 0.0, "sigma": 1, "a": 2, "b": 2},
     "StudentT": {"nu": 7, "mu": 0.0, "sigma": 1},
     "Triangular": {"lower": -2, "c": 0.0, "upper": 2.0},
     "TruncatedNormal": {"mu": 0.0, "sigma": 1, "lower": -2, "upper": 3.0},
@@ -120,4 +146,18 @@ init_vals = {
     "ZeroInflatedPoisson": {"psi": 0.8, "mu": 4.5},
     "Truncated": {"lower": -10, "upper": 10},
     "Censored": {"lower": -10, "upper": 10},
+    "Dirichlet": {"alpha": [1.0, 1.0, 1.0]},
 }
+
+
+def get_distributions(dist_names=None):
+    if dist_names is None:
+        all_distributions = modules["preliz.distributions"].__all__
+    else:
+        all_distributions = dist_names
+
+    distributions = []
+    for a_dist in all_distributions:
+        dist = getattr(modules["preliz.distributions"], a_dist)()
+        distributions.append(dist)
+    return distributions
