@@ -1,34 +1,43 @@
 import inspect
-import traceback
 import sys
+import traceback
 
 try:
     from IPython import get_ipython
-    from ipywidgets import FloatSlider, IntSlider, FloatText, IntText, Checkbox, ToggleButton
-    from pymc import sample_prior_predictive
+    from ipywidgets import (
+        Checkbox,
+        FloatSlider,
+        FloatText,
+        IntSlider,
+        IntText,
+        RadioButtons,
+        ToggleButton,
+    )
 except ImportError:
     pass
 
-from arviz import plot_kde, plot_ecdf, hdi, extract
-from arviz.stats.density_utils import _kde_linear
-import numpy as np
 import matplotlib.pyplot as plt
+import numpy as np
+from arviz_stats.base import array_stats
 from matplotlib import _pylab_helpers, get_backend
 from matplotlib.ticker import MaxNLocator
-from .logging import disable_pymc_sampling_logs
+
+from preliz.internal.rcparams import rcParams
 
 
-def plot_pointinterval(distribution, interval="hdi", levels=None, rotated=False, ax=None):
+def plot_pointinterval(distribution, interval=None, levels=None, rotated=False, ax=None):
     """
     Plot median as a dot and intervals as lines.
-    By defaults the intervals are HDI with 0.5 and 0.94 mass.
+
+    By defaults the intervals are HDI with 0.5 and the value in rcParams["stats.ci_prob"].
 
     Parameters
     ----------
     distribution : preliz distribution or array
     interval : str
-        Type of interval. Available options are highest density interval `"hdi"` (default),
+        Type of interval. Available options are highest density interval `"hdi"`,
         equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+        Defaults to the value in rcParams["stats.ci_kind"].
     levels : list
         Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
         For quantiles the number of elements should be 5, 3, 1 or 0
@@ -37,29 +46,37 @@ def plot_pointinterval(distribution, interval="hdi", levels=None, rotated=False,
         Whether to do the plot along the x-axis (default) or on the y-axis
     ax : matplotlib axis
     """
+    if isinstance(distribution, np.ndarray | list | tuple):
+        dist_type = "sample"
+    else:
+        dist_type = "preliz"
+
+    if interval is None:
+        interval = rcParams["stats.ci_kind"]
+
     if interval == "quantiles":
         if levels is None:
             levels = [0.05, 0.25, 0.5, 0.75, 0.95]
         elif len(levels) not in (5, 3, 1, 0):
             raise ValueError("levels should have 5, 3, 1 or 0 elements")
 
-        if isinstance(distribution, (np.ndarray, list, tuple)):
+        if dist_type == "sample":
             q_s = np.quantile(distribution, levels).tolist()
         else:
             q_s = distribution.ppf(levels).tolist()
 
     else:
         if levels is None:
-            levels = [0.5, 0.94]
+            levels = [0.5, rcParams["stats.ci_prob"]]
 
         elif len(levels) not in (2, 1):
             raise ValueError("levels should have 2 or 1 elements")
 
-        if isinstance(distribution, (np.ndarray, list, tuple)):
+        if dist_type == "sample":
             if interval == "hdi":
-                func = hdi
+                func = array_stats.hdi
             if interval == "eti":
-                func = eti
+                func = array_stats.eti
 
             q_tmp = np.concatenate([func(distribution, m) for m in levels])
             median = np.median(distribution)
@@ -80,43 +97,51 @@ def plot_pointinterval(distribution, interval="hdi", levels=None, rotated=False,
 
     q_s_size = len(q_s)
 
-    if rotated:
-        if q_s_size == 5:
-            ax.plot([0, 0], (q_s.pop(0), q_s.pop(-1)), "k", solid_capstyle="butt", lw=1.5)
-        if q_s_size > 2:
-            ax.plot([0, 0], (q_s.pop(0), q_s.pop(-1)), "k", solid_capstyle="butt", lw=4)
-        if q_s_size > 0:
-            ax.plot(0, q_s[0], "wo", mec="k")
+    if q_s_size == 5:
+        _plot_sub_iterval(q_s, lw=1.5, rotated=rotated, ax=ax)
+    if q_s_size > 2:
+        _plot_sub_iterval(q_s, lw=4, rotated=rotated, ax=ax)
+    if q_s_size > 0:
+        x, y = q_s[0], 0
+        if rotated:
+            x, y = y, x
+        ax.plot(x, y, "wo", mec="k")
+
+
+def _plot_sub_iterval(q_s, lw, rotated, ax):
+    lower, upper = q_s.pop(0), q_s.pop(-1)
+    if lower < upper:
+        x, y = (lower, upper), [0, 0]
+        if rotated:
+            x, y = y, x
+        ax.plot(x, y, "k", solid_capstyle="butt", lw=lw)
     else:
-        if q_s_size == 5:
-            ax.plot((q_s.pop(0), q_s.pop(-1)), [0, 0], "k", solid_capstyle="butt", lw=1.5)
-        if q_s_size > 2:
-            ax.plot((q_s.pop(0), q_s.pop(-1)), [0, 0], "k", solid_capstyle="butt", lw=4)
-
-        if q_s_size > 0:
-            ax.plot(q_s[0], 0, "wo", mec="k")
-
-
-def eti(distribution, mass):
-    lower = (1 - mass) / 2
-    return np.quantile(distribution, (lower, mass + lower))
+        x0, y0 = (lower, np.pi), [0, 0]
+        x1, y1 = (-np.pi, upper), [0, 0]
+        if rotated:
+            x0, y0 = y0, x0
+            x1, y1 = y1, x1
+        ax.plot(x0, y0, "k", solid_capstyle="butt", lw=lw)
+        ax.plot(x1, y1, "k", solid_capstyle="butt", lw=lw)
 
 
 def plot_pdfpmf(
-    dist, moments, pointinterval, interval, levels, support, legend, color, alpha, figsize, ax
+    dist,
+    moments,
+    pointinterval,
+    interval,
+    levels,
+    support,
+    baseline,
+    legend,
+    figsize,
+    ax,
+    kwargs,
 ):
+    if kwargs is None:
+        kwargs = {}
     ax = get_ax(ax, figsize)
-    if legend is not None:
-        label = repr_to_matplotlib(dist)
-
-        if moments is not None:
-            label += get_moments(dist, moments)
-
-        if legend == "title":
-            ax.set_title(label)
-            label = None
-    else:
-        label = None
+    label = set_label(dist, legend, moments, ax)
 
     x = dist.xvals(support)
     if dist.kind == "continuous":
@@ -131,11 +156,13 @@ def plot_pdfpmf(
             x = x[x != dist.support[1]]
 
         density = dist.pdf(x)
-        ax.axhline(0, color="0.8", ls="--", zorder=0)
-        p = ax.plot(x, density, label=label, color=color, alpha=alpha)
+        if baseline:
+            ax.axhline(0, color="0.8", ls="--", zorder=0)
+        p = ax.plot(x, density, label=label, **kwargs)
         ax.set_yticks([])
 
         if dist.__class__.__name__ == "Censored":
+            alpha = kwargs.get("alpha", 1)
             if disc_vals[0] is not None:
                 ax.vlines(lower, 0, disc_vals[0], ls="--", color=p[0].get_color(), alpha=alpha)
             if disc_vals[1] is not None:
@@ -143,45 +170,39 @@ def plot_pdfpmf(
 
     else:
         mass = dist.pdf(x)
-
+        alpha = kwargs.get("alpha", 1)
         if dist.__class__.__name__ in ["Categorical", "Bernoulli"]:
-            p = ax.plot(x, mass, "o", label=label, color=color, alpha=alpha)
+            p = ax.plot(x, mass, "o", label=label, **kwargs)
+            alpha = kwargs.get("alpha", 1)
             ax.vlines(x, 0, mass, ls="dotted", color=p[0].get_color(), alpha=alpha)
         else:
             x_c = np.linspace(x[0], x[-1], 1000)
             # we compute pmf at non-integer values to get a continuous curve
             mass_c = np.clip(dist.pdf(x_c), np.min(mass), np.max(mass))
 
-            p = ax.plot(x_c, mass_c, ls="dotted", color=color, alpha=alpha)
-            ax.plot(x, mass, "o", label=label, color=p[0].get_color(), alpha=alpha)
+            ls = kwargs.pop("ls", "dotted")
+            marker = kwargs.pop("marker", "o")
+            p = ax.plot(x_c, mass_c, ls=ls, **kwargs)
+            color = kwargs.get("color", p[0].get_color())
+            ax.plot(x, mass, ls="none", label=label, marker=marker, color=color)
 
         ax.xaxis.set_major_locator(MaxNLocator(integer=True))
-        ax.axhline(0, color="0.8", ls="--", zorder=0)
+        if baseline:
+            ax.axhline(0, color="0.8", ls="--", zorder=0)
 
     if pointinterval:
         plot_pointinterval(dist, interval, levels, ax=ax)
 
-    if legend == "legend":
-        side_legend(ax)
+    side_legend(legend, ax)
 
     return ax
 
 
-def plot_cdf(
-    dist, moments, pointinterval, interval, levels, support, legend, color, alpha, figsize, ax
-):
+def plot_cdf(dist, moments, pointinterval, interval, levels, support, legend, figsize, ax, kwargs):
+    if kwargs is None:
+        kwargs = {}
     ax = get_ax(ax, figsize)
-    if legend is not None:
-        label = repr_to_matplotlib(dist)
-
-        if moments is not None:
-            label += get_moments(dist, moments)
-
-        if legend == "title":
-            ax.set_title(label)
-            label = None
-    else:
-        label = None
+    label = set_label(dist, legend, moments, ax)
 
     ax.set_ylim(-0.05, 1.05)
     x = dist.xvals(support)
@@ -193,43 +214,35 @@ def plot_cdf(
         cdf = dist.cdf(x)
         ax.set_xlim(lower - 0.1, upper + 0.1)
         ax.xaxis.set_major_locator(MaxNLocator(integer=True))
-        ax.step(x, cdf, where="post", label=label, color=color, alpha=alpha)
+        ax.step(x, cdf, where="post", label=label, **kwargs)
     else:
         cdf = dist.cdf(x)
-        ax.plot(x, cdf, label=label, color=color, alpha=alpha)
+        ax.plot(x, cdf, label=label, **kwargs)
 
     if pointinterval:
         plot_pointinterval(dist, interval, levels, ax=ax)
 
-    if legend == "legend":
-        side_legend(ax)
+    side_legend(legend, ax)
+
     return ax
 
 
-def plot_ppf(dist, moments, pointinterval, interval, levels, legend, color, alpha, figsize, ax):
+def plot_ppf(dist, moments, pointinterval, interval, levels, legend, figsize, ax, kwargs):
+    if kwargs is None:
+        kwargs = {}
     ax = get_ax(ax, figsize)
-    if legend is not None:
-        label = repr_to_matplotlib(dist)
-
-        if moments is not None:
-            label += get_moments(dist, moments)
-
-        if legend == "title":
-            ax.set_title(label)
-            label = None
-    else:
-        label = None
+    label = set_label(dist, legend, moments, ax)
 
     x = np.linspace(0, 1, 1000)
-    ax.plot(x, dist.ppf(x), label=label, color=color, alpha=alpha)
+    ax.plot(x, dist.ppf(x), label=label, **kwargs)
     if dist.kind == "discrete":
         ax.yaxis.set_major_locator(MaxNLocator(integer=True))
 
     if pointinterval:
         plot_pointinterval(dist, interval, levels, rotated=True, ax=ax)
 
-    if legend == "legend":
-        side_legend(ax)
+    side_legend(legend, ax)
+
     return ax
 
 
@@ -243,10 +256,11 @@ def get_ax(ax, figsize):
     return ax
 
 
-def side_legend(ax):
-    bbox = ax.get_position()
-    ax.set_position([bbox.x0, bbox.y0, bbox.width, bbox.height])
-    ax.legend(loc="center left", bbox_to_anchor=(1, 0.5))
+def side_legend(legend, ax):
+    if isinstance(legend, str) and legend != "title":
+        bbox = ax.get_position()
+        ax.set_position([bbox.x0, bbox.y0, bbox.width, bbox.height])
+        ax.legend(loc="center left", bbox_to_anchor=(1, 0.5))
 
 
 def repr_to_matplotlib(distribution):
@@ -277,7 +291,6 @@ def get_moments(dist, moments):
 
 
 def get_slider(name, value, lower, upper):
-
     min_v, max_v, step = generate_range(value, lower, upper)
 
     if isinstance(value, float):
@@ -317,7 +330,6 @@ def generate_range(value, lower, upper):
 
 
 def get_boxes(name, value, lower, upper):
-
     if isinstance(value, float):
         text_type = FloatText
         step = 0.1
@@ -335,10 +347,10 @@ def get_boxes(name, value, lower, upper):
     return text
 
 
-def get_textboxes(signature, model):
+def get_textboxes(signature, model, kind_plot):
     textboxes = {}
     for name, param in signature.parameters.items():
-        if isinstance(param.default, (int, float)):
+        if isinstance(param.default, int | float):
             value = float(param.default)
         else:
             value = None
@@ -385,6 +397,24 @@ def get_textboxes(signature, model):
         disabled=False,
     )
 
+    textboxes["__set_ylim__"] = Checkbox(
+        value=False, description="set ylim", disabled=False, indent=False
+    )
+
+    textboxes["__y_min__"] = FloatText(
+        value=-10,
+        step=0.1,
+        description="y_min",
+        disabled=False,
+    )
+
+    textboxes["__y_max__"] = FloatText(
+        value=10,
+        step=0.1,
+        description="y_max",
+        disabled=False,
+    )
+
     textboxes["__resample__"] = ToggleButton(
         value=True,
         description="Resample",
@@ -393,6 +423,13 @@ def get_textboxes(signature, model):
         tooltip="Resample",
     )
 
+    textboxes["__kind__"] = RadioButtons(
+        options=["hist", "kde", "ecdf"],
+        value=kind_plot,
+        description="Kind",
+        disabled=False,
+    )
+
     return textboxes
 
 
@@ -426,62 +463,9 @@ def plot_decorator(func, iterations, kind_plot, references, plot_func):
     return looper
 
 
-def bambi_plot_decorator(func, iterations, kind_plot, references, plot_func):
-    def looper(*args, **kwargs):
-        kwargs.pop("__resample__")
-        x_min = kwargs.pop("__x_min__")
-        x_max = kwargs.pop("__x_max__")
-        if not kwargs.pop("__set_xlim__"):
-            x_min = None
-            x_max = None
-            auto = True
-        else:
-            auto = False
-
-        model = func(*args, **kwargs)
-        model.build()
-        with disable_pymc_sampling_logs():
-            idata = model.prior_predictive(iterations)
-        results = extract(idata, group="prior_predictive")[model.response_name].values.T
-
-        _, ax = plt.subplots()
-        ax.set_xlim(x_min, x_max, auto=auto)
-        if plot_func is None:
-            plot_repr(results, kind_plot, references, iterations, ax)
-        else:
-            plot_func(results, ax)
-
-    return looper
-
-
-def pymc_plot_decorator(func, iterations, kind_plot, references, plot_func):
-    def looper(*args, **kwargs):
-        kwargs.pop("__resample__")
-        x_min = kwargs.pop("__x_min__")
-        x_max = kwargs.pop("__x_max__")
-        if not kwargs.pop("__set_xlim__"):
-            x_min = None
-            x_max = None
-            auto = True
-        else:
-            auto = False
-        with func(*args, **kwargs) as model:
-            obs_name = model.observed_RVs[0].name
-            with disable_pymc_sampling_logs():
-                idata = sample_prior_predictive(samples=iterations)
-            results = extract(idata, group="prior_predictive")[obs_name].values.T
-        _, ax = plt.subplots()
-        ax.set_xlim(x_min, x_max, auto=auto)
-        if plot_func is None:
-            plot_repr(results, kind_plot, references, iterations, ax)
-        else:
-            plot_func(results, ax)
-
-    return looper
-
-
-def plot_repr(results, kind_plot, references, iterations, ax):
+def plot_repr(results, kind_plot, references, iterations, stats_kwargs, ax):
     alpha = max(0.01, 1 - iterations * 0.009)
+    stats_kwargs.setdefault("alpha", alpha)
 
     if kind_plot == "hist":
         if results[0].dtype.kind == "i":
@@ -490,31 +474,31 @@ def plot_repr(results, kind_plot, references, iterations, ax):
                 ax.set_xticks(bins + 0.5)
         else:
             bins = "auto"
-        ax.hist(
-            results.T,
-            alpha=alpha,
-            density=True,
-            color=["0.5"] * iterations,
-            bins=bins,
-            histtype="step",
-        )
-        ax.hist(
-            np.concatenate(results),
-            density=True,
-            bins=bins,
-            color="k",
-            ls="--",
-            histtype="step",
-        )
+
+        stats_kwargs.setdefault("bins", bins)
+        stats_kwargs.setdefault("density", True)
+        stats_kwargs.setdefault("histtype", "step")
+        stats_kwargs.setdefault("alpha", alpha)
+        stats_kwargs.setdefault("color", ["0.5"] * iterations)
+
+        ax.hist(results.T, **stats_kwargs)
+        stats_kwargs.pop("color")
+        stats_kwargs.pop("ls", None)
+        ax.hist(np.concatenate(results), color="k", ls="--", **stats_kwargs)
     elif kind_plot == "kde":
+        stats_kwargs.setdefault("color", "0.5")
         for result in results:
-            ax.plot(*_kde_linear(result, grid_len=100), "0.5", alpha=alpha)
-        ax.plot(*_kde_linear(np.concatenate(results), grid_len=100), "k--")
+            grid, pdf, _ = array_stats.kde(result)
+            ax.plot(grid, pdf, **stats_kwargs)
+        grid, pdf, _ = array_stats.kde(np.concatenate(results))
+        ax.plot(grid, pdf, "k--")
+
     elif kind_plot == "ecdf":
+        stats_kwargs.setdefault("color", "0.5")
         ax.plot(
             np.sort(results, axis=1).T,
             np.linspace(0, 1, len(results[0]), endpoint=False),
-            color="0.5",
+            **stats_kwargs,
         )
         a = np.concatenate(results)
         ax.plot(np.sort(a), np.linspace(0, 1, len(a), endpoint=False), "k--")
@@ -541,31 +525,27 @@ def plot_pp_samples(pp_samples, pp_samples_idxs, references, kind="pdf", sharex=
 
     for ax, idx in zip(axes, pp_samples_idxs):
         ax.clear()
-        plot_references(references, ax)
         ax.relim()
 
         sample = pp_samples[idx]
 
-        if sharex:
-            min_ = sample.min()
-            max_ = sample.max()
-            if min_ < x_lims[0]:
-                x_lims[0] = min_
-            if max_ > x_lims[1]:
-                x_lims[1] = max_
-
         if kind == "pdf":
-            plot_kde(sample, ax=ax, plot_kwargs={"color": "C0"})  # pylint:disable=no-member
+            plot_kde(sample, ax=ax, color="C0")
         elif kind == "hist":
-            bins, *_ = ax.hist(
-                sample, color="C0", bins="auto", alpha=0.5, density=True
-            )  # pylint:disable=no-member
+            bins, *_ = ax.hist(sample, color="C0", bins="auto", alpha=0.5, density=True)
             ax.set_ylim(-bins.max() * 0.05, None)
 
         elif kind == "ecdf":
-            plot_ecdf(sample, ax=ax, plot_kwargs={"color": "C0"})  # pylint:disable=no-member
+            ax.ecdf(sample, color="C0")
 
         plot_pointinterval(sample, ax=ax)
+        plot_references(references, ax)
+
+        if sharex:
+            min_, max_ = ax.get_xlim()
+            x_lims[0] = min(min_, x_lims[0])
+            x_lims[1] = max(max_, x_lims[1])
+
         ax.set_title(idx, alpha=0)
         ax.set_yticks([])
 
@@ -586,30 +566,29 @@ def plot_pp_mean(pp_samples, selected, references=None, kind="pdf", fig_pp_mean=
         ax_pp_mean = fig_pp_mean.axes[0]
 
     ax_pp_mean.clear()
+    ax_pp_mean.relim()
 
     if np.any(selected):
         sample = pp_samples[selected].ravel()
     else:
         sample = pp_samples.ravel()
 
-    plot_references(references, ax_pp_mean)
-
     if kind == "pdf":
         plot_kde(
-            sample, ax=ax_pp_mean, plot_kwargs={"color": "k", "linestyle": "--"}
-        )  # pylint:disable=no-member
+            sample,
+            ax=ax_pp_mean,
+            color="C0",
+            linestyle="--",
+        )
     elif kind == "hist":
-        bins, *_ = ax_pp_mean.hist(
-            sample, color="k", ls="--", bins="auto", alpha=0.5, density=True
-        )  # pylint:disable=no-member
+        bins, *_ = ax_pp_mean.hist(sample, color="k", ls="--", bins="auto", alpha=0.5, density=True)
         ax_pp_mean.set_ylim(-bins.max() * 0.05, None)
 
     elif kind == "ecdf":
-        plot_ecdf(
-            sample, ax=ax_pp_mean, plot_kwargs={"color": "k", "linestyle": "--"}
-        )  # pylint:disable=no-member
+        ax_pp_mean.ecdf(sample, color="k", linestyle="--")
 
     plot_pointinterval(sample, ax=ax_pp_mean)
+    plot_references(references, ax_pp_mean)
     ax_pp_mean.set_yticks([])
     fig_pp_mean.canvas.draw()
 
@@ -624,12 +603,17 @@ def plot_references(references, ax):
                 ax.text(ref, max_value * 0.2, label, rotation=90, bbox={"color": "w", "alpha": 0.5})
                 ax.axvline(ref, ls="--", color="0.5")
         else:
-            if isinstance(references, (float, int)):
+            if isinstance(references, float | int):
                 references = [references]
             for ref in references:
                 ax.axvline(ref, ls="--", color="0.5")
 
 
+def plot_kde(sample, ax, **kwargs):
+    grid, pdf, _ = array_stats.kde(sample)
+    ax.plot(grid, pdf, **kwargs)
+
+
 def check_inside_notebook(need_widget=False):
     shell = get_ipython()
     name = inspect.currentframe().f_back.f_code.co_name
@@ -640,10 +624,14 @@ def check_inside_notebook(need_widget=False):
             )
         if need_widget:
             shell_name = shell.__class__.__name__
-            if shell_name == "ZMQInteractiveShell" and "nbagg" not in get_backend():
+            if shell_name == "ZMQInteractiveShell" and get_backend().split("_")[-1].lower() not in [
+                "ipympl",
+                "widget",
+                "nbagg",
+            ]:
                 msg = f"To run {name}, you need use the magic `%matplotlib widget`"
                 raise RuntimeError(msg)
-    except Exception:  # pylint: disable=broad-except
+    except Exception:
         tb_as_str = traceback.format_exc()
         # Print only the last line of the traceback, which contains the error message
         print(tb_as_str.strip().rsplit("\n", maxsplit=1)[-1], file=sys.stdout)
@@ -667,9 +655,7 @@ def representations(fitted_dist, kind_plot, ax):
 
 
 def create_figure(figsize):
-    """
-    Initialize a matplotlib figure with one subplot
-    """
+    """Initialize a matplotlib figure with one subplot."""
     fig, axes = plt.subplots(1, 1, figsize=figsize, constrained_layout=True)
     axes.set_yticks([])
     fig.canvas.header_visible = False
@@ -680,11 +666,27 @@ def create_figure(figsize):
 
 
 def reset_dist_panel(ax, yticks):
-    """
-    Clean the distribution subplot
-    """
+    """Clean the distribution subplot."""
     ax.cla()
     if yticks:
         ax.set_yticks([])
     ax.relim()
     ax.autoscale_view()
+
+
+def set_label(dist, legend, moments, ax):
+    if legend is not None:
+        if isinstance(legend, str) and legend not in ["title", "legend"]:
+            label = legend
+        else:
+            label = repr_to_matplotlib(dist)
+
+            if moments is not None:
+                label += get_moments(dist, moments)
+
+            if legend == "title":
+                ax.set_title(label)
+                label = None
+    else:
+        label = None
+    return label
