@@ -1,7 +1,7 @@
 """Prior predictive check assistant."""
 
-import logging
 import ast
+import warnings
 from random import shuffle
 
 try:
@@ -12,22 +12,26 @@ import matplotlib.pyplot as plt
 import numpy as np
 from scipy.spatial import KDTree
 
-
-from ..internal.plot_helper import (
+from preliz.distributions import Normal
+from preliz.distributions.distributions import Distribution
+from preliz.internal.plot_helper import (
     check_inside_notebook,
-    plot_pp_samples,
     plot_pp_mean,
+    plot_pp_samples,
 )
-from ..internal.parser import get_prior_pp_samples, from_preliz, from_bambi
-from ..internal.predictive_helper import back_fitting, select_prior_samples
-from ..distributions import Normal
-from ..distributions.distributions import Distribution
-
-_log = logging.getLogger("preliz")
+from preliz.internal.predictive_helper import back_fitting_ppa, select_prior_samples
+from preliz.ppls.agnostic import from_preliz, get_prior_pp_samples
+from preliz.ppls.bambi_io import from_bambi
 
 
 def ppa(
-    fmodel, draws=2000, references=0, boundaries=(-np.inf, np.inf), target=None, engine="preliz"
+    fmodel,
+    draws=2000,
+    references=0,
+    boundaries=(-np.inf, np.inf),
+    target=None,
+    new_families=True,
+    engine="preliz",
 ):
     """
     Prior predictive check assistant.
@@ -49,14 +53,19 @@ def ppa(
         Target distribution. The first shown distributions will be selected to be as close
         as possible to `target`. Available options are, a PreliZ distribution or a 2-tuple with
         the first element representing the mean and the second the standard deviation.
+    new_families : bool
+        If True, the method will return the best fitting distribution from a set of common
+        distributions
     engine : str
         Library used to define the model. Either `preliz` or `bambi`. Defaults to `preliz`
     """
     check_inside_notebook(need_widget=True)
 
-    _log.info(""""This is an experimental method under development, use with caution.""")
+    warnings.warn(""""This is an experimental method under development, use with caution.""")
 
-    filter_dists = FilterDistribution(fmodel, draws, references, boundaries, target, engine)
+    filter_dists = FilterDistribution(
+        fmodel, draws, references, boundaries, target, new_families, engine
+    )
     filter_dists()
 
     output = widgets.Output()
@@ -64,7 +73,7 @@ def ppa(
     with output:
         references_widget = widgets.Text(
             value=str(references),
-            placeholder="Int, Float or tuple",
+            placeholder="Int, Float, tuple or dict",
             description="references: ",
             disabled=False,
             layout=widgets.Layout(width="230px", margin="0 20px 0 0"),
@@ -90,14 +99,26 @@ def ppa(
 
         def kind_(_):
             kind = radio_buttons_kind.value
+            try:
+                filter_dists.references = ast.literal_eval(references_widget.value)
+            except (ValueError, SyntaxError):
+                filter_dists.references = None
+
             plot_pp_samples(
                 filter_dists.pp_samples,
                 filter_dists.display_pp_idxs,
-                ast.literal_eval(references_widget.value),
+                filter_dists.references,
                 kind,
                 check_button_sharex.value,
                 filter_dists.fig,
             )
+            plot_pp_mean(
+                filter_dists.pp_samples,
+                list(filter_dists.selected),
+                filter_dists.references,
+                kind,
+                filter_dists.fig_pp_mean,
+            )
 
         references_widget.observe(kind_, names=["value"])
 
@@ -122,24 +143,24 @@ def ppa(
     controls = widgets.VBox([button_carry_on, button_return_prior])
     plot_combine = widgets.VBox([radio_buttons_kind, check_button_sharex])
 
-    display(  # pylint:disable=undefined-variable
-        widgets.HBox([references_widget, plot_combine, controls, output])
-    )
+    display(widgets.HBox([references_widget, plot_combine, controls, output]))  # noqa:F821
 
 
-class FilterDistribution:  # pylint:disable=too-many-instance-attributes
-    def __init__(self, fmodel, draws, references, boundaries, target, engine):
+class FilterDistribution:
+    def __init__(self, fmodel, draws, references, boundaries, target, new_families, engine):
         self.fmodel = fmodel
         self.source = ""  # string representation of the model
         self.draws = draws
         self.references = references
         self.boundaries = boundaries
         self.target = target
+        self.new_families = new_families
         self.engine = engine
         self.pp_samples = None  # prior predictive samples
         self.prior_samples = None  # prior samples used for backfitting
         self.display_pp_idxs = None  # indices of the pp_samples to be displayed
         self.pp_octiles = None  # octiles computed from pp_samples
+        self.ref_octiles = None  # octiles computed from the target distribution
         self.kdt = None  # KDTree used to find similar distributions
         self.model = None  # parsed model used for backfitting
         self.clicked = []  # axes clicked by the user
@@ -154,7 +175,6 @@ class FilterDistribution:  # pylint:disable=too-many-instance-attributes
         self.axes = None  # axes used to display the pp_samples
 
     def __call__(self):
-
         if self.engine == "preliz":
             variables, self.model = from_preliz(self.fmodel)
         elif self.engine == "bambi":
@@ -170,7 +190,9 @@ class FilterDistribution:  # pylint:disable=too-many-instance-attributes
         self.pp_octiles, self.kdt = self.compute_octiles()
         self.display_pp_idxs = self.initialize_subsamples(self.target)
         self.fig, self.axes = plot_pp_samples(
-            self.pp_samples, self.display_pp_idxs, self.references
+            self.pp_samples,
+            self.display_pp_idxs,
+            self.references,
         )
         self.fig_pp_mean = plot_pp_mean(self.pp_samples, self.selected, self.references)
 
@@ -180,15 +202,14 @@ class FilterDistribution:  # pylint:disable=too-many-instance-attributes
         elif isinstance(self.target, Distribution):
             ref_sample = self.target.rvs(self.pp_samples.shape[1])
 
-        self.pp_samples = np.vstack([ref_sample, self.pp_samples])
+        self.ref_octiles = np.quantile(ref_sample, [0.125, 0.25, 0.375, 0.5, 0.625, 0.75, 0.875])
 
     def compute_octiles(self):
         """
-        Compute the octiles for the prior predictive samples. This is used to find
-        similar distributions using a KDTree.
+        Compute the octiles for the prior predictive samples.
 
-        We have empirically found that octiles are a good choice, but this could
-        be the consequence of limited testing.
+        This is used to find similar distributions using a KDTree. We have empirically found that
+        octiles are a good choice, but this could be the consequence of limited testing.
         """
         pp_octiles = np.quantile(
             self.pp_samples, [0.125, 0.25, 0.375, 0.5, 0.625, 0.75, 0.875], axis=1
@@ -217,10 +238,11 @@ class FilterDistribution:  # pylint:disable=too-many-instance-attributes
 
         self.shown.update(shown_list)
 
+        rng = np.random.default_rng(235)
         # If we have not seen all the samples yet, we collect more
         if len(self.shown) != self.draws:
             if target is None:
-                new = np.random.choice(list(set(range(0, len(self.pp_octiles))) - self.shown))
+                new = rng.choice(list(set(range(0, len(self.pp_octiles))) - self.shown))
                 pp_idxs_to_display.append(new)
 
                 for _ in range(8):
@@ -235,14 +257,14 @@ class FilterDistribution:  # pylint:disable=too-many-instance-attributes
                     if new != self.draws:
                         pp_idxs_to_display.append(new)
             else:
-                new = 0
+                new = -1
                 pp_idxs_to_display.append(new)
 
                 for _ in range(9):
                     nearest_neighbor = 2
                     while new in pp_idxs_to_display:
                         distance, new = self.kdt.query(
-                            self.pp_octiles[pp_idxs_to_display[-1]], [nearest_neighbor], workers=-1
+                            self.ref_octiles, [nearest_neighbor], workers=-1
                         )
                         new = new.item()
                         nearest_neighbor += 1
@@ -260,6 +282,7 @@ class FilterDistribution:  # pylint:disable=too-many-instance-attributes
     def keep_sampling(self):
         """
         Find distribution similar to the ones in `choices`, but not already shown.
+
         If `choices` is empty return an empty selection.
         """
         if self.choices:
@@ -288,7 +311,7 @@ class FilterDistribution:  # pylint:disable=too-many-instance-attributes
 
     def collect_more_samples(self):
         """
-        Automatically extend the set of user selected distributions
+        Automatically extend the set of user selected distributions.
 
         If the user has selected at least two distributions we automatically extend the selection
         by adding all the distributions that are close to the selected ones. To do so we use
@@ -328,9 +351,7 @@ class FilterDistribution:  # pylint:disable=too-many-instance-attributes
             self.shown.update(new)
 
     def carry_on(self, kind, sharex):
-        """
-        Collect user input and update the plot.
-        """
+        """Collect user input and update the plot."""
         self.fig.suptitle("")
 
         if self.clicked:
@@ -365,13 +386,12 @@ class FilterDistribution:  # pylint:disable=too-many-instance-attributes
             self.fig.canvas.draw()
 
     def on_return_prior(self):
-
         selected = list(self.selected)
 
         if len(selected) > 4:
             subsample = select_prior_samples(selected, self.prior_samples, self.model)
 
-            string, _ = back_fitting(self.model, subsample, new_families=False)
+            string, _ = back_fitting_ppa(self.model, subsample, new_families=self.new_families)
 
             self.fig.clf()
             plt.text(0.05, 0.5, string, fontsize=14)
