@@ -1,34 +1,42 @@
-import pytest
 import numpy as np
-
+import pytest
 from numpy.testing import assert_allclose, assert_almost_equal
 
-
 from preliz import maxent
 from preliz.distributions import (
     AsymmetricLaplace,
     Beta,
+    # Bernoulli, maxent is not useful for Bernoulli distribution as we only have two states
+    BetaBinomial,
     BetaScaled,
+    Binomial,
     Cauchy,
     ChiSquared,
+    DiscreteUniform,
+    DiscreteWeibull,
     ExGaussian,
     Exponential,
     Gamma,
+    Geometric,
     Gumbel,
     HalfCauchy,
     HalfNormal,
     HalfStudentT,
+    HyperGeometric,
     InverseGamma,
     Kumaraswamy,
     Laplace,
     Logistic,
+    LogitNormal,
     LogLogistic,
     LogNormal,
-    LogitNormal,
     Moyal,
+    NegativeBinomial,
     Normal,
     Pareto,
+    Poisson,
     Rice,
+    ScaledInverseChiSquared,
     SkewNormal,
     SkewStudentT,
     StudentT,
@@ -38,15 +46,6 @@ from preliz.distributions import (
     VonMises,
     Wald,
     Weibull,
-    # Bernoulli, maxent is not useful for Bernoulli distribution as we only have two states
-    BetaBinomial,
-    Binomial,
-    DiscreteUniform,
-    DiscreteWeibull,
-    Geometric,
-    HyperGeometric,
-    NegativeBinomial,
-    Poisson,
     ZeroInflatedBinomial,
     ZeroInflatedNegativeBinomial,
     ZeroInflatedPoisson,
@@ -103,15 +102,16 @@ from preliz.distributions import (
         (Normal(), -1, 1, 0.683, (-np.inf, np.inf), (0, 1)),
         (Normal(), 10, 12, 0.99, (-np.inf, np.inf), (11, 0.388)),
         (Normal(mu=0.5), -1, 1, 0.8, (-np.inf, np.inf), (0.581)),
-        (Pareto(), 1, 4, 0.9, (1, np.inf), (1.660, 1)),
+        (Pareto(), 1, 4, 0.9, (1, np.inf), (1.694, 0.997)),
         (Pareto(m=2), 1, 4, 0.9, (2, np.inf), (3.321)),
         (Rice(), 0, 4, 0.7, (0, np.inf), (0, 2.577)),
         (Rice(), 1, 10, 0.9, (0, np.inf), (3.454, 3.734)),
         (Rice(nu=4), 0, 6, 0.9, (0, np.inf), (1.402)),
+        (ScaledInverseChiSquared(), 0.1, 4, 0.9, (0, np.inf), (4.831, 1.253)),
         (SkewNormal(), -2, 10, 0.9, (-np.inf, np.inf), (4, 3.647, 0)),
-        (SkewNormal(mu=-1), -2, 10, 0.9, (-np.inf, np.inf), (6.292, 4.903)),
-        (SkewStudentT(), -1, 1, 0.9, (-np.inf, np.inf), (-0.095, 0.52, 3.488, 3.112)),
-        (SkewStudentT(mu=0.7, sigma=0.4), -1, 1, 0.9, (-np.inf, np.inf), (2.004, 5.212)),
+        (SkewNormal(mu=-1), -2, 10, 0.9, (-np.inf, np.inf), (6.293, 4.908)),
+        (SkewStudentT(), -1, 1, 0.9, (-np.inf, np.inf), (0.010, 0.522, 3.264, 3.305)),
+        (SkewStudentT(mu=0.7, sigma=0.4), -1, 1, 0.9, (-np.inf, np.inf), (2.004, 5.214)),
         (StudentT(), -1, 1, 0.683, (-np.inf, np.inf), (99.999, 0, 0.994)),
         (StudentT(nu=7), -1, 1, 0.683, (-np.inf, np.inf), (0, 0.928)),
         (
@@ -160,9 +160,9 @@ from preliz.distributions import (
         (Poisson(), 0, 3, 0.7, (0, np.inf), (2.763)),
         (ZeroInflatedBinomial(), 1, 10, 0.9, (0, 10), (0.902, 9.0, 0.485)),
         (ZeroInflatedBinomial(psi=0.7), 1, 10, 0.7, (0, 11), (10, 0.897)),
-        (ZeroInflatedNegativeBinomial(), 2, 15, 0.8, (0, np.inf), (1.0, 9.862, 3.429)),
-        (ZeroInflatedNegativeBinomial(psi=0.9), 2, 15, 0.8, (0, np.inf), (8.965, 6.174)),
-        (ZeroInflatedPoisson(), 0, 3, 0.7, (0, np.inf), (0.8445, 3.010)),
+        (ZeroInflatedNegativeBinomial(), 2, 15, 0.8, (0, np.inf), (1.0, 9.864, 3.432)),
+        (ZeroInflatedNegativeBinomial(psi=0.9), 2, 15, 0.8, (0, np.inf), (9.011, 6.300)),
+        (ZeroInflatedPoisson(), 0, 3, 0.7, (0, np.inf), (0.847, 3.005)),
         (ZeroInflatedPoisson(psi=0.8), 0, 3, 0.7, (0, np.inf), (3.099)),
     ],
 )
@@ -180,5 +180,26 @@ def test_maxent(dist, lower, upper, mass, support, result):
     assert_allclose(dist.opt.x, result, atol=0.001)
 
 
+def test_maxent_fixed_stats():
+    dist = Beta()
+    maxent(dist, 0.1, 0.7, 0.94, fixed_stat=("mode", 0.3))
+    assert_almost_equal(dist.mode(), 0.3)
+    assert_almost_equal(dist.params, (2.7, 5.1), 1)
+
+    dist = Gamma()
+    maxent(dist, 0, 3, 0.8, fixed_stat=("mode", 2))
+    assert_almost_equal(dist.mode(), 2)
+    assert_almost_equal(dist.params, (7.2, 3.1), 1)
+
+    dist = Gamma()
+    maxent(dist, 0.1, 3, 0.8, fixed_stat=("mean", 2))
+    assert_almost_equal(dist.mean(), 2)
+    assert_almost_equal(dist.params, (2.1, 1.0), 1)
+
+    with pytest.raises(ValueError, match="fixed_stat should be one of the following"):
+        dist = Gamma()
+        maxent(dist, 0, 3, 0.8, fixed_stat=("bad", 2))
+
+
 def test_maxent_plot():
     maxent(Normal(), plot_kwargs={"support": "restricted", "pointinterval": True})
