@@ -1,33 +1,42 @@
+import matplotlib.pyplot as plt
 import pytest
 from numpy.testing import assert_allclose
 
 import preliz as pz
-
-
 from preliz.distributions import (
     AsymmetricLaplace,
+    Bernoulli,
     Beta,
+    BetaBinomial,
     BetaScaled,
+    Binomial,
     Cauchy,
     ChiSquared,
+    DiscreteUniform,
+    DiscreteWeibull,
     ExGaussian,
     Exponential,
     Gamma,
+    Geometric,
     Gumbel,
     HalfCauchy,
     HalfNormal,
     HalfStudentT,
+    HyperGeometric,
     InverseGamma,
     Kumaraswamy,
     Laplace,
     Logistic,
+    LogitNormal,
     LogLogistic,
     LogNormal,
-    LogitNormal,
     Moyal,
+    NegativeBinomial,
     Normal,
     Pareto,
+    Poisson,
     Rice,
+    ScaledInverseChiSquared,
     SkewNormal,
     SkewStudentT,
     StudentT,
@@ -37,15 +46,6 @@ from preliz.distributions import (
     VonMises,
     Wald,
     Weibull,
-    Bernoulli,
-    BetaBinomial,
-    Binomial,
-    DiscreteUniform,
-    DiscreteWeibull,
-    Geometric,
-    HyperGeometric,
-    NegativeBinomial,
-    Poisson,
     ZeroInflatedBinomial,
     ZeroInflatedNegativeBinomial,
     ZeroInflatedPoisson,
@@ -79,6 +79,7 @@ from preliz.distributions import (
         (Normal, (0, 1)),
         (Pareto, (5, 1)),
         (Rice, (0, 2)),
+        (ScaledInverseChiSquared, (10, 1.0)),
         (SkewNormal, (0, 1, -6)),
         (SkewStudentT, (0, 1, 2, 2)),
         (StudentT, (4, 0, 1)),
@@ -120,7 +121,7 @@ def test_auto_recover(distribution, params):
                 tol = 1
             else:
                 tol = 0.5
-            pz.mle([dist], sample)
+            pz.mle([dist], sample, plot=0)
             assert_allclose(dist.params, params, atol=tol)
             break
         except AssertionError:
@@ -132,13 +133,26 @@ def test_auto_recover(distribution, params):
 def test_recover_right():
     dists = [Normal(), Gamma(), Poisson()]
     sample = Normal(0, 1).rvs(10000)
-    pz.mle(dists, sample)
-    assert dists[0].__class__.__name__ == "Normal"
+    idx, ax = pz.mle(dists, sample, plot=0)
+    assert len(idx) == len(dists)
+    assert idx[0] == 0
+    assert ax is None
 
+    dists = [pz.Normal(), pz.Gamma(), pz.Poisson()]
+    sample = pz.Normal(10, 0.5).rvs((2, 10000), random_state=123)
+    idx, ax = pz.mle(dists, sample, plot=0)
+    all(d.params[0].size == 2 for d in dists)
+    assert idx[0] == 0
+    assert ax is None
+
+    plt.figure()
     sample = Gamma(2, 10).rvs(10000)
-    pz.mle(dists, sample)
-    assert dists[1].__class__.__name__ == "Gamma"
+    idx, ax = pz.mle(dists, sample)
+    assert idx[0] == 1
+    assert len(ax.get_legend().legend_handles) == 1
 
+    plt.figure()
     sample = Poisson(10).rvs(10000)
-    pz.mle(dists, sample)
-    assert dists[2].__class__.__name__ == "Poisson"
+    idx, ax = pz.mle(dists, sample, plot=3)
+    assert idx[0] == 2
+    assert len(ax.get_legend().legend_handles) == 3
