@@ -1,12 +1,11 @@
-import logging
+import warnings
 
 import numpy as np
 
-from ..distributions import Normal
-from ..internal.distribution_helper import valid_distribution
-from ..internal.optimization import relative_error, optimize_quartile, get_fixed_params
-
-_log = logging.getLogger("preliz")
+from preliz.distributions.normal import Normal
+from preliz.internal.distribution_helper import valid_distribution
+from preliz.internal.optimization import get_fixed_params, optimize_quartile, relative_error
+from preliz.internal.rcparams import rcParams
 
 
 def quartile(
@@ -14,7 +13,7 @@ def quartile(
     q1=-1,
     q2=0,
     q3=1,
-    plot=True,
+    plot=None,
     plot_kwargs=None,
     ax=None,
 ):
@@ -33,14 +32,14 @@ def quartile(
     q3 : float
         Third quartile, i.e 0.75 of the mass is below this point.
     plot : bool
-        Whether to plot the distribution, and lower and upper bounds. Defaults to True.
+        Whether to plot the distribution, and lower and upper bounds. Defaults to None,
+        which results in the value of rcParams["plots.show_plot"] being used.
     plot_kwargs : dict
         Dictionary passed to the method ``plot_pdf()`` of ``distribution``.
     ax : matplotlib axes
 
     Returns
     -------
-
     dict: dict with the parameters of the distribution
     axes: matplotlib axes (only if `plot=True`)
 
@@ -61,7 +60,6 @@ def quartile(
         :context: close-figs
         :include-source: true
 
-        >>>
         >>> import preliz as pz
         >>> pz.style.use('preliz-doc')
         >>> pz.quartile(pz.Gamma(), 3, 6, 8)
@@ -73,7 +71,6 @@ def quartile(
         :context: close-figs
         :include-source: true
 
-        >>>
         >>> import preliz as pz
         >>> pz.style.use('preliz-doc')
         >>> pz.quartile(pz.HalfStudentT(nu=7), 2, 9, 12)
@@ -81,6 +78,9 @@ def quartile(
     """
     valid_distribution(distribution)
 
+    if plot is None:
+        plot = rcParams["plots.show_plot"]
+
     if plot_kwargs is None:
         plot_kwargs = {}
 
@@ -103,7 +103,7 @@ def quartile(
     # Heuristic to provide an initial guess for the optimization step
     # We obtain those guesses by first approximating the mean and standard deviation
     # from the quartiles and then use those values for moment matching
-    distribution._fit_moments(mean=q2, sigma=(q3 - q1) / 1.35)  # pylint:disable=protected-access
+    distribution._fit_moments(mean=q2, sigma=(q3 - q1) / 1.35)
 
     opt = optimize_quartile(distribution, quartiles, none_idx, fixed)
 
@@ -112,9 +112,12 @@ def quartile(
     r_error, _ = relative_error(distribution, q1, q3, 0.5)
 
     if r_error > 0.01:
-        _log.info(
-            "The expected masses are 0.25, 0.5, 0.75\n The computed ones are: %.2g, %.2g, %.2g",
-            *distribution.cdf(quartiles)
+        computed_masses = distribution.cdf(quartiles).astype(float)
+        warnings.warn(
+            f"\nThe expected masses are 0.25, 0.5, 0.75\n"
+            f"The computed ones are: {computed_masses[0]:.2g}, "
+            f"{computed_masses[1]:.2g}, {computed_masses[2]:.2g}",
+            stacklevel=2,
         )
 
     if plot:
